//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_Clipboard.h"
#include "Moa_Window.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_clipboard {
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Initialize()
{
	try {
		FMessageWindow = new TMessageWindow();
		MessageWindow->OnMessage = MessageWindowMessage;
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::Uninitialize()
{
	delete FMessageWindow;
	FMessageWindow = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::MessageWindowMessage(TObject* ASender, TMessage& AMessage, bool& AHandled)
{
	switch(AMessage.Msg) {
	case WM_CHANGECBCHAIN:
		AHandled = true;
		WMChangeCbChain(AMessage);
		break;
	case WM_DRAWCLIPBOARD:
		AHandled = true;
		WMDrawClipboard(AMessage);
		break;
	default:
		AHandled = false;
		break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMChangeCbChain(TMessage& AMessage)
{
	if(reinterpret_cast<HWND>(AMessage.WParam) == NextHandle) {
		FNextHandle = reinterpret_cast<HWND>(AMessage.LParam);
	}
	else {
		::SendMessage(NextHandle, AMessage.Msg, AMessage.WParam, AMessage.LParam);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::WMDrawClipboard(TMessage& AMessage)
{
	DoChange();
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::DoChange()
{
	if(OnChange != NULL) {
		OnChange(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TClipboardWatcher::SetEnabled(bool AEnabled)
{
	if((AEnabled ? 1 : 0) == (Enabled ? 1 : 0)) {
		return;
	}
	if(AEnabled) {
		::SetLastError(NO_ERROR);
		HWND LNextHandle = ::SetClipboardViewer(MessageWindow->Handle);
		if(::GetLastError() != NO_ERROR) {
			throw EClipboardError("The program could not enable the clipboard watcher.");
		}
		FNextHandle = LNextHandle;
	}
	else {
		::ChangeClipboardChain(MessageWindow->Handle, NextHandle); // No way to detect errors
		FNextHandle = NULL;
	}
	FEnabled = AEnabled;
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::TClipboardWatcher() :
	TObject(),
	FMessageWindow(NULL),
	FNextHandle(NULL),
	FEnabled(false),
	FOnChange(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TClipboardWatcher::~TClipboardWatcher()
{
	Enabled = false;
	Uninitialize();
}
//---------------------------------------------------------------------------
} // End Moa_clipboard
//---------------------------------------------------------------------------

