//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_AutoPointer.h"
#include "Moa_Command.h"
#include "Moa_List.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moa_command {
//---------------------------------------------------------------------------
__fastcall TCommand::TCommand() :
	TObject(),
	FResults()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TCommand::~TCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
void __fastcall TInvoker::Initialize()
{
	try {
		FCommands = new TObjectList();
		FUndoCommands = new TObjectList();
		FRedoCommands = new TObjectList();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Uninitialize()
{
	delete FRedoCommands;
	FRedoCommands = NULL;

	delete FUndoCommands;
	FUndoCommands = NULL;

	delete FCommands;
	FCommands = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::ExecuteDone()
{
	if(NextCommand->Results.Contains(crCancelled)) {
		Commands->Clear();
		return;
	}
	TAutoPointer<TCommand> LCommand = static_cast<TCommand*>(TObjectListHelper(Commands).PopFirst());
	if(!LCommand->Results.Contains(crCompleted)) {
		Commands->Clear();
	}
	if(LCommand->Results.Contains(crUndoable)) {
		UndoCommands->Add(LCommand.Release());
	}
	RedoCommands->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::UndoDone()
{
	if(NextUndoCommand->Results.Contains(crCancelled)) {
		return;
	}
	TAutoPointer<TCommand> LCommand = static_cast<TCommand*>(TObjectListHelper(UndoCommands).PopLast());
	if(!LCommand->Results.Contains(crCompleted)) {
		UndoCommands->Clear();
	}
	if(!LCommand->Results.Contains(crUndoable)) {
		UndoCommands->Clear();
	}
	if(LCommand->Results.Contains(crRedoable)) {
		RedoCommands->Add(LCommand.Release());
	}
	else {
		RedoCommands->Clear();
	}
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::RedoDone()
{
	if(NextRedoCommand->Results.Contains(crCancelled)) {
		return;
	}
	TAutoPointer<TCommand> LCommand = static_cast<TCommand*>(TObjectListHelper(RedoCommands).PopLast());
	if(!LCommand->Results.Contains(crCompleted)) {
		RedoCommands->Clear();
	}
	if(!LCommand->Results.Contains(crRedoable)) {
		RedoCommands->Clear();
	}
	if(LCommand->Results.Contains(crUndoable)) {
		UndoCommands->Add(LCommand.Release());
	}
	else {
		UndoCommands->Clear();
	}
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextCommand()
{
	return static_cast<TCommand*>(Commands->First());
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextUndoCommand()
{
	return static_cast<TCommand*>(UndoCommands->Last());
}
//---------------------------------------------------------------------------
TCommand* __fastcall TInvoker::GetNextRedoCommand()
{
	return static_cast<TCommand*>(RedoCommands->Last());
}
//---------------------------------------------------------------------------
__fastcall TInvoker::TInvoker() :
	TObject(),
	FCommands(NULL),
	FUndoCommands(NULL),
	FRedoCommands(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TInvoker::~TInvoker()
{
	Uninitialize();
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Execute()
{
	NextCommand->Results = TCommandResults();
	try {
		NextCommand->Execute();
	}
	catch(...) {
		ExecuteDone();
		throw;
	}
	ExecuteDone();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanExecute()
{
	return Commands->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Undo()
{
	NextUndoCommand->Results = TCommandResults();
	try {
		NextUndoCommand->Undo();
	}
	catch(...) {
		UndoDone();
		throw;
	}
	UndoDone();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanUndo()
{
	return UndoCommands->Count > 0;
}
//---------------------------------------------------------------------------
void __fastcall TInvoker::Redo()
{
	NextRedoCommand->Results = TCommandResults();
	try {
		NextRedoCommand->Redo();
	}
	catch(...) {
		RedoDone();
		throw;
	}
	RedoDone();
}
//---------------------------------------------------------------------------
bool __fastcall TInvoker::CanRedo()
{
	return RedoCommands->Count > 0;
}
//---------------------------------------------------------------------------
} // End namespace Moa_command
//---------------------------------------------------------------------------

