//---------------------------------------------------------------------------
#ifndef Moa_FileH
#define Moa_FileH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include "Moa_Enumerator.h"
//---------------------------------------------------------------------------
namespace Moa_file {
//---------------------------------------------------------------------------
class EFileError : public Exception
{
public:
	__fastcall EFileError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TPath
{
public:
	static const bool ValidPathCharTable[256];
	static const bool ValidNameCharTable[256];
	static const bool WildcardCharTable[256];
	static const char DriveSep;
	static const char DirectorySep;
	static const char ExtensionSep;
private:
	AnsiString FPath;
	AnsiString __fastcall GetPath() const;
	void __fastcall SetPath(const AnsiString& APath);
	AnsiString __fastcall GetDrivePart() const;
	void __fastcall SetDrivePart(const AnsiString& ADrivePart);
	AnsiString __fastcall GetDrive() const;
	void __fastcall SetDrive(const AnsiString& ADrive);
	AnsiString __fastcall GetRoot() const;
	void __fastcall SetRoot(const AnsiString& ARoot);
	AnsiString __fastcall GetDirectoryPart() const;
	void __fastcall SetDirectoryPart(const AnsiString& ADirectoryPart);
	AnsiString __fastcall GetDirectory() const;
	void __fastcall SetDirectory(const AnsiString& ADirectory);
	AnsiString __fastcall GetName() const;
	void __fastcall SetName(const AnsiString& AName);
	AnsiString __fastcall GetBaseName() const;
	void __fastcall SetBaseName(const AnsiString& ABaseName);
	AnsiString __fastcall GetExtensionPart() const;
	void __fastcall SetExtensionPart(const AnsiString& AExtensionPart);
	AnsiString __fastcall GetExtension() const;
	void __fastcall SetExtension(const AnsiString& AExtension);
	int __fastcall GetLength() const;
	void __fastcall SetLength(int ALength);
protected:
public:
	static int __fastcall Compare(const AnsiString& APath1, const AnsiString& APath2);
	static TPath __fastcall Compose(const AnsiString& ADirectory, const AnsiString& AName);
	static TPath __fastcall ComposeName(const AnsiString& ABaseName, const AnsiString& AExtension);
	static bool __fastcall IsValidPathChar(char AChar);
	static bool __fastcall IsValidNameChar(char AChar);
	static bool __fastcall IsWildcardChar(char AChar);
	__fastcall TPath();
	__fastcall TPath(const TPath& AOther);
	__fastcall TPath(const AnsiString& APath);
	virtual __fastcall ~TPath();
	TPath& __fastcall operator =(const TPath& AOther);
	TPath __fastcall operator +(const TPath& AOther) const;
	TPath& __fastcall operator +=(const TPath& AOther);
	bool __fastcall operator ==(const TPath& AOther) const;
	bool __fastcall operator !=(const TPath& AOther) const;
	bool __fastcall operator <(const TPath& AOther) const;
	bool __fastcall operator <=(const TPath& AOther) const;
	bool __fastcall operator >(const TPath& AOther) const;
	bool __fastcall operator >=(const TPath& AOther) const;
	__fastcall operator AnsiString() const;
	virtual TPath& __fastcall Expand();
	virtual TPath& __fastcall ExpandUnc();
	virtual TPath& __fastcall StartWith(char AChar);
	virtual TPath& __fastcall StartWithout(char AChar);
	virtual TPath& __fastcall EndWith(char AChar);
	virtual TPath& __fastcall EndWithout(char AChar);
	virtual void __fastcall Process(TStrings* AComponents) const;
	virtual TPath& __fastcall Push(const AnsiString& AComponent);
	virtual AnsiString __fastcall Pop();
	virtual AnsiString __fastcall Peek() const;
	virtual TPath& __fastcall PushExtension(const AnsiString& AExtension);
	virtual AnsiString __fastcall PopExtension();
	virtual AnsiString __fastcall PeekExtension() const;
	virtual TPath& __fastcall MakeAbsolute(const AnsiString& ADirectory);
	virtual TPath& __fastcall MakeRelative(const AnsiString& ADirectory);
	virtual TPath& __fastcall MakeUpperCase();
	virtual TPath& __fastcall MakeLowerCase();
	virtual bool __fastcall IsAbsolute() const;
	virtual bool __fastcall IsRelative() const;
	virtual bool __fastcall IsRoot() const;
	virtual bool __fastcall IsMask() const;
	virtual bool __fastcall IsDot() const;
	virtual bool __fastcall IsDotDot() const;
	virtual bool __fastcall IsValidPath() const;
	virtual bool __fastcall IsValidName() const;
	virtual bool __fastcall StartsWith(char AChar) const;
	virtual bool __fastcall EndsWith(char AChar) const;
	virtual int __fastcall Compare(const TPath& AOther) const;
	__property AnsiString Path = {read=GetPath, write=SetPath};
	__property AnsiString DrivePart = {read=GetDrivePart, write=SetDrivePart};
	__property AnsiString Drive = {read=GetDrive, write=SetDrive};
	__property AnsiString Root = {read=GetRoot, write=SetRoot};
	__property AnsiString DirectoryPart = {read=GetDirectoryPart, write=SetDirectoryPart};
	__property AnsiString Directory = {read=GetDirectory, write=SetDirectory};
	__property AnsiString Name = {read=GetName, write=SetName};
	__property AnsiString BaseName = {read=GetBaseName, write=SetBaseName};
	__property AnsiString ExtensionPart = {read=GetExtensionPart, write=SetExtensionPart};
	__property AnsiString Extension = {read=GetExtension, write=SetExtension};
	__property int Length = {read=GetLength, write=SetLength};
};
//---------------------------------------------------------------------------
class TDeviceNames
{
private:
	static const char* Table[];
	static const int TableSize;
	__fastcall TDeviceNames(const TDeviceNames& AOther); // Forbidden
	TDeviceNames& __fastcall operator =(const TDeviceNames& AOther); // Forbidden
protected:
	virtual AnsiString __fastcall GetName(int AIndex) const;
	virtual int __fastcall GetCount() const;
public:
	__fastcall TDeviceNames();
	virtual __fastcall ~TDeviceNames();
	virtual int __fastcall Find(const AnsiString& AName);
	__property AnsiString Names[int AIndex] = {read=GetName};
	__property int Count = {read=GetCount};
};
//---------------------------------------------------------------------------
class TFile
{
private:
	AnsiString FPath;
	AnsiString __fastcall GetLongPath() const;
	AnsiString __fastcall GetLongName() const;
	AnsiString __fastcall GetShortPath() const;
	AnsiString __fastcall GetShortName() const;
	DWORDLONG __fastcall GetSize() const;
	DWORD __fastcall GetAttributes() const;
	void __fastcall SetAttributes(DWORD AAttributes);
	FILETIME __fastcall GetCreationTime() const;
	void __fastcall SetCreationTime(const FILETIME& ACreationTime);
	FILETIME __fastcall GetLastWriteTime() const;
	void __fastcall SetLastWriteTime(const FILETIME& ALastWriteTime);
	FILETIME __fastcall GetLastAccessTime() const;
	void __fastcall SetLastAccessTime(const FILETIME& ALastAccessTime);
	WIN32_FIND_DATA __fastcall GetData() const;
protected:
public:
	__fastcall TFile();                                        
	__fastcall TFile(const TFile& AOther);
	__fastcall TFile(const AnsiString& APath);
	virtual __fastcall ~TFile();
	bool __fastcall TryGetLongPath(AnsiString& ALongPath) const;
	bool __fastcall TryGetLongName(AnsiString& ALongName) const;
	bool __fastcall TryGetShortPath(AnsiString& AShortPath) const;
	bool __fastcall TryGetShortName(AnsiString& AShortName) const;
	bool __fastcall TryGetSize(DWORDLONG& ASize) const;
	bool __fastcall TryGetAttributes(DWORD& AAttributes) const;
	bool __fastcall TrySetAttributes(DWORD AAttributes);
	void __fastcall GetTimes(FILETIME* ACrTime, FILETIME* ALaTime, FILETIME* ALwTime) const;
	bool __fastcall TryGetTimes(FILETIME* ACrTime, FILETIME* ALaTime, FILETIME* ALwTime) const;
	void __fastcall SetTimes(const FILETIME* ACrTime, const FILETIME* ALaTime, const FILETIME* ALwTime);
	bool __fastcall TrySetTimes(const FILETIME* ACrTime, const FILETIME* ALaTime, const FILETIME* ALwTime);
	bool __fastcall TryGetCreationTime(FILETIME& ACreationTime) const;
	bool __fastcall TrySetCreationTime(const FILETIME& ACreationTime);
	bool __fastcall TryGetLastWriteTime(FILETIME& ALastWriteTime) const;
	bool __fastcall TrySetLastWriteTime(const FILETIME& ALastWriteTime);
	bool __fastcall TryGetLastAccessTime(FILETIME& ALastAccessTime) const;
	bool __fastcall TrySetLastAccessTime(const FILETIME& ALastAccessTime);
	bool __fastcall TryGetData(WIN32_FIND_DATA& AData) const;
	__property AnsiString Path = {read=FPath};
	__property AnsiString LongPath = {read=GetLongPath};
	__property AnsiString LongName = {read=GetLongName};
	__property AnsiString ShortPath = {read=GetShortPath};
	__property AnsiString ShortName = {read=GetShortName};
	__property DWORDLONG Size = {read=GetSize};
	__property DWORD Attributes = {read=GetAttributes, write=SetAttributes};
	__property FILETIME CreationTime = {read=GetCreationTime, write=SetCreationTime};
	__property FILETIME LastWriteTime = {read=GetLastWriteTime, write=SetLastWriteTime};
	__property FILETIME LastAccessTime = {read=GetLastAccessTime, write=SetLastAccessTime};
	__property WIN32_FIND_DATA Data = {read=GetData};
};
//---------------------------------------------------------------------------
class TFiles
{
private:
	AnsiString __fastcall GetCurrentDirectory() const;
	void __fastcall SetCurrentDirectory(const AnsiString& ACurrentDirectory);
protected:
public:
	__fastcall TFiles();
	virtual __fastcall ~TFiles();
	virtual AnsiString __fastcall CreateTemporaryFile(const AnsiString& ADirectory, const AnsiString& APrefix);
	virtual void __fastcall Move(const AnsiString& ASrcPath, const AnsiString& ADstPath);
	virtual bool __fastcall TryMove(const AnsiString& ASrcPath, const AnsiString& ADstPath);
	virtual void __fastcall Copy(const AnsiString& ASrcPath, const AnsiString& ADstPath, bool AOverwirte);
	virtual bool __fastcall TryCopy(const AnsiString& ASrcPath, const AnsiString& ADstPath, bool AOverwirte);
	virtual void __fastcall Delete(const AnsiString& APath);
	virtual bool __fastcall TryDelete(const AnsiString& APath);
	virtual void __fastcall Rename(const AnsiString& APath, const AnsiString& AName);
	virtual bool __fastcall TryRename(const AnsiString& APath, const AnsiString& AName);
	virtual void __fastcall RemoveDirectory(const AnsiString& APath);
	virtual bool __fastcall TryRemoveDirectory(const AnsiString& APath);
	virtual bool __fastcall Exists(const AnsiString& APath) const;
	virtual bool __fastcall DriveExists(const AnsiString& APath) const;
	virtual bool __fastcall DirectoryExists(const AnsiString& APath) const;
	virtual bool __fastcall RegularFileExists(const AnsiString& APath) const;
	__property AnsiString CurrentDirectory = {read=GetCurrentDirectory, write=SetCurrentDirectory};
};
//---------------------------------------------------------------------------
class TSystemDirs
{
private:
protected:
	virtual AnsiString __fastcall GetTempDir() const;
	virtual AnsiString __fastcall GetSystemDir() const;
	virtual AnsiString __fastcall GetWindowsDir() const;
public:
	__fastcall TSystemDirs();
	virtual __fastcall ~TSystemDirs();
	__property AnsiString TempDir = {read=GetTempDir};
	__property AnsiString SystemDir = {read=GetSystemDir};
	__property AnsiString WindowsDir = {read=GetWindowsDir};
};
//---------------------------------------------------------------------------
class TFileEnumerator : public TEnumerator<WIN32_FIND_DATA>
{
private:
	enum TState
	{
		stNone,
		stBusy,
		stOver,
	};
	HANDLE FHandle;
	AnsiString FPath;
	WIN32_FIND_DATA FData;
	TState FState;
	bool __fastcall FindFirst();
	bool __fastcall FindNext();
	void __fastcall Finish();
	void __fastcall SetState(TState AState);
	__property HANDLE Handle = {read=FHandle};
	__property AnsiString Path = {read=FPath};
	__property WIN32_FIND_DATA Data = {read=FData};
	__property TState State = {read=FState, write=SetState};
protected:
	virtual WIN32_FIND_DATA __fastcall GetValue();
public:
	__fastcall TFileEnumerator(const AnsiString& APath);
	virtual __fastcall ~TFileEnumerator();
	virtual bool __fastcall GoToNext();
	virtual void __fastcall Reset();
};
//---------------------------------------------------------------------------
class TPathList : public TStringList
{
private:
protected:
	virtual int __fastcall CompareStrings(const AnsiString S1, const AnsiString S2);
public:
	__fastcall TPathList();
	virtual __fastcall ~TPathList();
};
//---------------------------------------------------------------------------
} // End namespace Moa_file
using namespace Moa_file;
//---------------------------------------------------------------------------
#endif

