//---------------------------------------------------------------------------
#ifndef Moa_ListViewModelH
#define Moa_ListViewModelH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
namespace Moa_listviewmodel {
//---------------------------------------------------------------------------
class EListViewModelError : public Exception
{
public:
	__fastcall EListViewModelError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TListViewModel;
typedef void __fastcall (__closure * TListViewModelInsertEvent)(TObject* ASender, int AIndex);
typedef void __fastcall (__closure * TListViewModelDeleteEvent)(TObject* ASender, int AIndex);
typedef int __fastcall (__closure * TListViewModelSortCompareMethod)(TListViewModel* AControl, int AIndex1, int AIndex2);
//---------------------------------------------------------------------------
struct TListViewModelSortItem
{
	TListViewModel* Control;
	int Index;
	TListViewModelSortCompareMethod CompareMethod;
};
//---------------------------------------------------------------------------
struct TListViewModelOrderItem
{
	int OldIndex;
	int NewIndex;
};
typedef DynamicArray<TListViewModelOrderItem> TListViewModelOrder;
//---------------------------------------------------------------------------
class TListViewModelEntry : public TObject
{
private:
	void* FValue;
	UINT FState;
protected:
public:
	__fastcall TListViewModelEntry();
	virtual __fastcall ~TListViewModelEntry();
	__property void* Value = {read=FValue, write=FValue};
	__property UINT State = {read=FState, write=FState};
};
//---------------------------------------------------------------------------
class TListViewModelData : public TObject
{
private:
	TListViewModel* FOwner;
	TObjectList* FEntries;
	bool FMultiSelect;
	int FFocusIndex;
	int FSelCount;
	int FSelIndex;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
public:
	__fastcall TListViewModelData(TListViewModel* AOwner);
	virtual __fastcall ~TListViewModelData();
	__property TListViewModel* Owner = {read=FOwner};
	__property TObjectList* Entries = {read=FEntries};
	__property bool MultiSelect = {read=FMultiSelect, write=FMultiSelect};
	__property int FocusIndex = {read=FFocusIndex, write=FFocusIndex};
	__property int SelCount = {read=FSelCount, write=FSelCount};
	__property int SelIndex = {read=FSelIndex, write=FSelIndex};
};
//---------------------------------------------------------------------------
class TListViewModelItem : public TObject
{
private:
	TListViewModelData* FData;
	int FIndex;
	void* __fastcall GetValue();
	void __fastcall SetValue(void* AValue);
	bool __fastcall GetFocused();
	void __fastcall SetFocused(bool AFocused);
	bool __fastcall GetSelected();
	void __fastcall SetSelected(bool ASelected);
	UINT __fastcall GetState();
	void __fastcall SetState(UINT AState);
protected:
	__property TListViewModelData* Data = {read=FData};
public:
	__fastcall TListViewModelItem(TListViewModelData* AData);
	virtual __fastcall ~TListViewModelItem();
	__property int Index = {read=FIndex, write=FIndex};
	__property void* Value = {read=GetValue, write=SetValue};
	__property bool Focused = {read=GetFocused, write=SetFocused};
	__property bool Selected = {read=GetSelected, write=SetSelected};
	__property UINT State = {read=GetState, write=SetState};
};
//---------------------------------------------------------------------------
class TListViewModel : public TObject
{
public:
	static const UINT StateMask;
private:
	TListView* FControl;
	TListViewModelData* FData;
	TListViewModelItem* FItem;
	TListViewModelInsertEvent FOnInserting;
	TListViewModelInsertEvent FOnInsert;
	TListViewModelDeleteEvent FOnDeleting;
	TListViewModelDeleteEvent FOnDelete;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
	void __fastcall DoInserting(int AIndex);
	void __fastcall DoInsert(int AIndex);
	void __fastcall DoDeleting(int AIndex);
	void __fastcall DoDelete(int AIndex);
	TListViewModelItem* __fastcall GetItem(int AIndex);
	int __fastcall GetCount();
	void __fastcall SetCount(int ACount);
	int __fastcall GetCapacity();
	void __fastcall SetCapacity(int ACapacity);
	int __fastcall GetFocusIndex();
	void __fastcall SetFocusIndex(int AFocusIndex);
	int __fastcall GetSelCount();
	int __fastcall GetSelIndex();
	void __fastcall SetSelIndex(int ASelIndex);
	bool __fastcall GetMultiSelect();
	void __fastcall SetMultiSelect(bool AMultiSelect);
protected:
	__property TListViewModelData* Data = {read=FData};
public:
	__fastcall TListViewModel(TListView* AControl);
	virtual __fastcall ~TListViewModel();
	virtual int __fastcall Add();
	virtual void __fastcall Insert(int AIndex);
	virtual void __fastcall DeleteSelected();
	virtual void __fastcall Delete(int AIndex);
	virtual void __fastcall Clear();
	virtual void __fastcall SelectAll();
	virtual void __fastcall ClearSelection();
	virtual void __fastcall ReverseSelection();
	virtual void __fastcall Sort(TListViewModelSortCompareMethod ACompareMethod);
	virtual void __fastcall Invert();
	virtual void __fastcall Exchange(int AIndex1, int AIndex2);
	virtual void __fastcall MoveSelected(int ADirection);
	virtual void __fastcall Move(int AOldIndex, int ANewIndex);
	virtual void __fastcall Reorder(TListViewModelOrder AOrder);
	virtual int __fastcall FindByValue(void* AValue);
	virtual void __fastcall Update(bool AUpdateState);
	virtual void __fastcall UpdateControl(bool AUpdateState);
	__property TListView* Control = {read=FControl};
	__property TListViewModelItem* Items[int AIndex] = {read=GetItem};
	__property int Count = {read=GetCount, write=SetCount};
	__property int Capacity = {read=GetCapacity, write=SetCapacity};
	__property int FocusIndex = {read=GetFocusIndex, write=SetFocusIndex};
	__property int SelCount = {read=GetSelCount};
	__property int SelIndex = {read=GetSelIndex, write=SetSelIndex};
	__property bool MultiSelect = {read=GetMultiSelect, write=SetMultiSelect};
	__property TListViewModelInsertEvent OnInserting = {read=FOnInserting, write=FOnInserting};
	__property TListViewModelInsertEvent OnInsert = {read=FOnInsert, write=FOnInsert};
	__property TListViewModelDeleteEvent OnDeleting = {read=FOnDeleting, write=FOnDeleting};
	__property TListViewModelDeleteEvent OnDelete = {read=FOnDelete, write=FOnDelete};
};
//---------------------------------------------------------------------------
} // End namespace Moa_listviewmodel
using namespace Moa_listviewmodel;
//---------------------------------------------------------------------------
#endif

