//---------------------------------------------------------------------------
#ifndef Moa_TaskH
#define Moa_TaskH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_task {
//---------------------------------------------------------------------------
class ETaskError : public Exception
{
public:
	__fastcall ETaskError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
class TTaskThread : public TThread
{
private:
	TNotifyEvent FOnExecute;
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall DoExecute();
public:
	__fastcall TTaskThread(bool ACreateSuspended);
	virtual __fastcall ~TTaskThread();
	virtual void __fastcall CallSynchronize(TThreadMethod AMethod);
	__property TNotifyEvent OnExecute = {read=FOnExecute, write=FOnExecute};
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TTaskMethod)(void* AData);
typedef void __fastcall (__closure * TTaskExceptionEvent)(TObject* ASender, Exception* AException);
//---------------------------------------------------------------------------
class TTask : public TObject
{
private:
	TTaskThread* FThread;
	TTaskMethod FSyncMethod;
	void* FSyncData;
	bool FCancelled;
	bool FFreeOnTerminate;
	TNotifyEvent FOnTerminate;
	TTaskExceptionEvent FOnException;
	void __fastcall SynchronizeInMainThread();
	void __fastcall DoTerminateInMainThread(void* AData);
	void __fastcall DoExceptionInMainThread(void* AData);
	void __fastcall ThreadExecute(TObject* ASender);
	__property TTaskMethod SyncMethod = {read=FSyncMethod, write=FSyncMethod};
	__property void* SyncData = {read=FSyncData, write=FSyncData};
protected:
	virtual void __fastcall Execute() = 0;
	virtual void __fastcall DoTerminate();
	virtual void __fastcall DoException(Exception* AException);
public:
	__fastcall TTask();
	virtual __fastcall ~TTask();
	virtual void __fastcall Perform();
	virtual void __fastcall Cancel();
	virtual void __fastcall Synchronize(TTaskMethod AMethod, void* AData);
	__property TTaskThread* Thread = {read=FThread};
	__property bool Cancelled = {read=FCancelled};
	__property bool FreeOnTerminate = {read=FFreeOnTerminate, write=FFreeOnTerminate};
	__property TNotifyEvent OnTerminate = {read=FOnTerminate, write=FOnTerminate};
	__property TTaskExceptionEvent OnException = {read=FOnException, write=FOnException};
};
//---------------------------------------------------------------------------
} // End namespace Moa_task
using namespace Moa_task;
//---------------------------------------------------------------------------
#endif

