//---------------------------------------------------------------------------
#ifndef Moa_WindowH
#define Moa_WindowH
//---------------------------------------------------------------------------
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moa_window {
//---------------------------------------------------------------------------
class EWindowError : public Exception
{
public:
	__fastcall EWindowError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
typedef void __fastcall (__closure * TMessageWindowMessageEvent)(TObject* ASender, TMessage& AMessage, bool& AHandled);
//---------------------------------------------------------------------------
class TMessageWindow : public TObject
{
private:
	HWND FHandle;
	TMessageWindowMessageEvent FOnMessage;
	void __fastcall Initialize();
	void __fastcall Uninitialize();
protected:
	virtual void __fastcall HandleMessage(TMessage& AMessage);
	virtual bool __fastcall DoMessage(TMessage& AMessage);
public:
	__fastcall TMessageWindow();
	virtual __fastcall ~TMessageWindow();
	__property HWND Handle = {read=FHandle};
	__property TMessageWindowMessageEvent OnMessage = {read=FOnMessage, write=FOnMessage};
};
//---------------------------------------------------------------------------
class TWindow
{
private:
	HWND FHandle;
protected:
	virtual HWND __fastcall GetParent() const;
	virtual void __fastcall SetParent(HWND AParent);
	virtual HWND __fastcall GetOwner() const;
	virtual HWND __fastcall GetWindow(UINT ARelationship) const;
	virtual bool __fastcall GetEnabled() const;
	virtual void __fastcall SetEnabled(bool AEnabled);
	virtual bool __fastcall GetVisible() const;
	virtual void __fastcall SetVisible(bool AVisible);
	virtual LONG __fastcall GetLongAttr(int AIndex) const;
	virtual void __fastcall SetLongAttr(int AIndex, LONG ALongAttr);
	virtual WORD __fastcall GetWordAttr(int AIndex) const;
	virtual void __fastcall SetWordAttr(int AIndex, WORD AWordAttr);
public:
	__fastcall TWindow(HWND AHandle);
	__fastcall TWindow(const TWindow& AOther);
	virtual __fastcall ~TWindow();
	TWindow& __fastcall operator =(const TWindow& AOther);
	virtual bool __fastcall Exists() const;
	virtual bool __fastcall HasAncestor(HWND AHandle) const;
	virtual bool __fastcall HasDescendant(HWND AHandle) const;
	__property HWND Handle = {read=FHandle, write=FHandle};
	__property HWND Parent = {read=GetParent, write=SetParent};
	__property HWND Owner = {read=GetOwner};
	__property HWND Windows[UINT ARelationship] = {read=GetWindow};
	__property bool Enabled = {read=GetEnabled, write=SetEnabled};
	__property bool Visible = {read=GetVisible, write=SetVisible};
	__property LONG LongAttrs[int AIndex] = {read=GetLongAttr, write=SetLongAttr};
	__property WORD WordAttrs[int AIndex] = {read=GetWordAttr, write=SetWordAttr};
};
//---------------------------------------------------------------------------
class TTopLevelWindow : public TWindow
{
private:
	virtual TPoint __fastcall GetMinPos() const;
	virtual void __fastcall SetMinPos(TPoint AMinPos);
	virtual TPoint __fastcall GetMaxPos() const;
	virtual void __fastcall SetMaxPos(TPoint AMaxPos);
	virtual TRect __fastcall GetNormalBounds() const;
	virtual void __fastcall SetNormalBounds(TRect ANormalBounds);
	virtual TRect __fastcall GetBounds() const;
	virtual void __fastcall SetBounds(TRect ABounds);
	virtual UINT __fastcall GetState() const;
	virtual void __fastcall SetState(UINT AState);
	virtual WINDOWPLACEMENT __fastcall GetPlacement() const;
	virtual void __fastcall SetPlacement(const WINDOWPLACEMENT& APlacement);
protected:
public:
	__fastcall TTopLevelWindow(HWND AHandle);
	__fastcall TTopLevelWindow(const TTopLevelWindow& AOther);
	virtual __fastcall ~TTopLevelWindow();
	TTopLevelWindow& __fastcall operator =(const TTopLevelWindow& AOther);
	__property TPoint MinPos = {read=GetMinPos, write=SetMinPos};
	__property TPoint MaxPos = {read=GetMaxPos, write=SetMaxPos};
	__property TRect NormalBounds = {read=GetNormalBounds, write=SetNormalBounds};
	__property TRect Bounds = {read=GetBounds, write=SetBounds};
	__property UINT State = {read=GetState, write=SetState};
	__property WINDOWPLACEMENT Placement = {read=GetPlacement, write=SetPlacement};
};
//---------------------------------------------------------------------------
class TTopLevelWindows
{
private:
	virtual HWND __fastcall GetForegroundWindow() const;
	virtual void __fastcall SetForegroundWindow(HWND AForegroundWindow);
protected:
public:
	__fastcall TTopLevelWindows();
	__fastcall TTopLevelWindows(const TTopLevelWindows& AOther);
	virtual __fastcall ~TTopLevelWindows();
	TTopLevelWindows& __fastcall operator =(const TTopLevelWindows& AOther);
	__property HWND ForegroundWindow = {read=GetForegroundWindow, write=SetForegroundWindow};
};
//---------------------------------------------------------------------------
} // namespace Moa_window
using namespace Moa_window;
//---------------------------------------------------------------------------
#endif

