//---------------------------------------------------------------------------
// Version: 2.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Moa_File.h"
#include "Moa_List.h"
#include "Oren_ListEdit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Oren_listedit {
//---------------------------------------------------------------------------
int __fastcall TItemUtils::ComparePath(const AnsiString& APath1, const AnsiString& APath2)
{
	return AnsiCompareStr(APath1, APath2);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareType(Oren_rename::TFileType AType1, Oren_rename::TFileType AType2)
{
	return static_cast<int>(AType1) - static_cast<int>(AType2);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareResult(TRenameResult AResult1, TRenameResult AResult2)
{
	return static_cast<int>(AResult1) - static_cast<int>(AResult2);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByDirectory(void* AItem1, void* AItem2)
{
	return ComparePath(
		static_cast<TRenameItem*>(AItem1)->Directory,
		static_cast<TRenameItem*>(AItem2)->Directory);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByOldName(void* AItem1, void* AItem2)
{
	return ComparePath(
		static_cast<TRenameItem*>(AItem1)->OldName,
		static_cast<TRenameItem*>(AItem2)->OldName);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByOldPath(void* AItem1, void* AItem2)
{
	int LResult = ComparePath(
		static_cast<TRenameItem*>(AItem1)->Directory,
		static_cast<TRenameItem*>(AItem2)->Directory);
	if(LResult == 0) {
		LResult = ComparePath(
			static_cast<TRenameItem*>(AItem1)->OldName,
			static_cast<TRenameItem*>(AItem2)->OldName);
	}
	return LResult;
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByNewName(void* AItem1, void* AItem2)
{
	return ComparePath(
		static_cast<TRenameItem*>(AItem1)->NewName,
		static_cast<TRenameItem*>(AItem2)->NewName);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByNewPath(void* AItem1, void* AItem2)
{
	int LResult = ComparePath(
		static_cast<TRenameItem*>(AItem1)->Directory,
		static_cast<TRenameItem*>(AItem2)->Directory);
	if(LResult == 0) {
		LResult = ComparePath(
			static_cast<TRenameItem*>(AItem1)->NewName,
			static_cast<TRenameItem*>(AItem2)->NewName);
	}
	return LResult;
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByType(void* AItem1, void* AItem2)
{
	return ComparePath(
		static_cast<TRenameItem*>(AItem1)->Type,
		static_cast<TRenameItem*>(AItem2)->Type);
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::CompareByResult(void* AItem1, void* AItem2)
{
	return ComparePath(
		static_cast<TRenameItem*>(AItem1)->Result,
		static_cast<TRenameItem*>(AItem2)->Result);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TItemUtils::DescribePath(const AnsiString& APath)
{
	return APath;
}
//---------------------------------------------------------------------------
AnsiString __fastcall TItemUtils::DescribeType(Oren_rename::TFileType AType)
{
	switch(AType) {
	case Oren_rename::ftDirectory:
		return "Directory";
	case Oren_rename::ftRegularFile:
		return "Regular file";
	default:
		throw EListEditError("The type is not supported.");
	}
}
//---------------------------------------------------------------------------
AnsiString __fastcall TItemUtils::DescribeResult(TRenameResult AResult)
{
	switch(AResult) {
	case reNone:
		return "None";
	case reSucceeded:
		return "Succeeded";
	case reFailed:
		return "Failed";
	case reSkipped:
		return "Skipped";
	default:
		throw EListEditError("The result is not supported.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::DetermineImageIndex(TRenameItem* AItem)
{
	switch(AItem->Type) {
	case Oren_rename::ftDirectory:
		return 1;
	case Oren_rename::ftRegularFile:
		return 0;
	default:
		throw EListEditError("The type is not supported.");
	}
}
//---------------------------------------------------------------------------
int __fastcall TItemUtils::DetermineStateIndex(TRenameItem* AItem)
{
	switch(AItem->Result) {
	case reNone:
		return 2;
	case reSucceeded:
		return 3;
	case reFailed:
		return 4;
	case reSkipped:
		return 5;
	default:
		throw EListEditError("The result is not supported.");
	}
}
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
__fastcall TRenameList::TRenameList(TListViewModel* AControl) :
	FControl(AControl)
{
	// Do nothimg
}
//---------------------------------------------------------------------------
__fastcall TRenameList::~TRenameList()
{
	// Do nothimg
}
//---------------------------------------------------------------------------
int __fastcall TRenameList::Add(TRenameItem* AItem)
{
	int LIndex = Control->Count;
	Insert(LIndex, AItem);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall TRenameList::Insert(int AIndex, TRenameItem* AItem)
{
	Control->Insert(AIndex);
	Control->Items[AIndex]->Value = AItem;
	Control->Items[AIndex]->State = LVIS_FOCUSED | LVIS_SELECTED;
}
//---------------------------------------------------------------------------
int __fastcall TRenameList::Remove(TRenameItem* AItem)
{
	int LIndex = Control->FindByValue(AItem);
	if(LIndex < 0) {
		throw EListEditError("The program cannot remove the item.");
	}
	Delete(LIndex);
	return LIndex;
}
//---------------------------------------------------------------------------
void __fastcall TRenameList::Delete(int AIndex)
{
	Control->Delete(AIndex);
}
//---------------------------------------------------------------------------
} // End namespace Oren_listedit
//---------------------------------------------------------------------------

