//---------------------------------------------------------------------------
// Version: 2.1.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Clipbrd.hpp>
#include "Moa_AutoPointer.h"
#include "Oren_CapitalizeDialog.h"
#include "Oren_FreeEditDialog.h"
#include "Oren_InsertDialog.h"
#include "Oren_ListEdit.h"
#include "Oren_NameEdit.h"
#include "Oren_NameEditCommand.h"
#include "Oren_NameHandler.h"
#include "Oren_NumbersDialog.h"
#include "Oren_OverwriteDialog.h"
#include "Oren_PartDialog.h"
#include "Oren_ReplaceDialog.h"
#include "Oren_UrlEncodingDialog.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Oren_namehandler {
//---------------------------------------------------------------------------
__fastcall TNameHandler::TNameHandler(TListViewModel* AControl, TInvoker* AInvoker, TObjectList* AItems) :
	THandler(AControl, AInvoker, AItems)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TNameHandler::~TNameHandler()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Copy()
{
	if(OrenPartDialog == NULL) {
		OrenPartDialog = new TOrenPartDialog(Application);
	}
	if(!OrenPartDialog->Execute()) {
		Abort();
	}
	Control->Update(true);
	TAutoPointer<TStringList> LLines = new TStringList();
	LLines->Capacity = Control->SelCount;
	for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
		if(!Control->Items[LI]->Selected) {
			continue;
		}
		--LRest;
		TRenameName LName = static_cast<TRenameItem*>(Control->Items[LI]->Value)->NewName;
		LName.Name = LName.Parts[OrenPartDialog->Values.Part];
		LLines->Add(LName.Name);
	}
	Clipboard()->AsText = LLines->Text;
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Insert()
{
	if(OrenInsertDialog == NULL) {
		OrenInsertDialog = new TOrenInsertDialog(Application);
	}
	if(!OrenInsertDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		TAutoPointer<TStrings> LLines = new TStringList();
		OrenInsertDialog->Values.Text.GetSelected().Produce(Control->SelCount, LLines);
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			TRenameName LPart = LName.Parts[OrenInsertDialog->Values.Part];
			LPart.Insert(OrenInsertDialog->Values.Position, LLines->Strings[LCommand->Invoker->Commands->Count]);
			LName.Parts[OrenInsertDialog->Values.Part] = LPart;
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Overwrite()
{
	if(OrenOverwriteDialog == NULL) {
		OrenOverwriteDialog = new TOrenOverwriteDialog(Application);
	}
	if(!OrenOverwriteDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		TAutoPointer<TStrings> LLines = new TStringList();
		OrenOverwriteDialog->Values.Text.GetSelected().Produce(Control->SelCount, LLines);
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			LName.Parts[OrenOverwriteDialog->Values.Part] = LLines->Strings[LCommand->Invoker->Commands->Count];
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Replace()
{
	if(OrenReplaceDialog == NULL) {
		OrenReplaceDialog = new TOrenReplaceDialog(Application);
	}
	if(!OrenReplaceDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		TAutoPointer<TStrings> LLines = new TStringList();
		OrenReplaceDialog->Values.NewText.GetSelected().Produce(Control->SelCount, LLines);
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			LName.Replace(
				OrenReplaceDialog->Values.OldText,
				LLines->Strings[LCommand->Invoker->Commands->Count],
				OrenReplaceDialog->Values.MatchCase);
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Capitalize()
{
	if(OrenCapitalizeDialog == NULL) {
		OrenCapitalizeDialog = new TOrenCapitalizeDialog(Application);
	}
	if(!OrenCapitalizeDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			TRenameName LPart = LName.Parts[OrenCapitalizeDialog->Values.Part];
			LPart.Capitalize(OrenCapitalizeDialog->Values.Case);
			LName.Parts[OrenCapitalizeDialog->Values.Part] = LPart;
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::UrlEncode()
{
	if(OrenUrlEncodingDialog == NULL) {
		OrenUrlEncodingDialog = new TOrenUrlEncodingDialog(Application);
	}
	if(!OrenUrlEncodingDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			TRenameName LPart = LName.Parts[OrenUrlEncodingDialog->Values.Part];
			LPart.CharEncode(OrenUrlEncodingDialog->Values.Encoding);
			LPart.UrlEncode();
			LName.Parts[OrenUrlEncodingDialog->Values.Part] = LPart;
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::UrlDecode()
{
	if(OrenUrlEncodingDialog == NULL) {
		OrenUrlEncodingDialog = new TOrenUrlEncodingDialog(Application);
	}
	if(!OrenUrlEncodingDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		Control->Update(true);
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			TRenameName LPart = LName.Parts[OrenUrlEncodingDialog->Values.Part];
			LPart.UrlDecode();
			LPart.CharDecode(OrenUrlEncodingDialog->Values.Encoding);
			LName.Parts[OrenUrlEncodingDialog->Values.Part] = LPart;
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::FreeEdit()
{
	if(OrenPartDialog == NULL) {
		OrenPartDialog = new TOrenPartDialog(Application);
	}
	if(!OrenPartDialog->Execute()) {
		Abort();
	}
	if(OrenFreeEditDialog == NULL) {
		OrenFreeEditDialog = new TOrenFreeEditDialog(Application);
	}
	Control->Update(true);
	{
		TOrenFreeEditDialogValues LValues;
		{
			TAutoPointer<TStrings> LLines = new TStringList();
			LLines->Capacity = Control->SelCount;
			for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
				if(!Control->Items[LI]->Selected) {
					continue;
				}
				--LRest;
				TRenameName LName = static_cast<TRenameItem*>(Control->Items[LI]->Value)->NewName;
				LName.Name = LName.Parts[OrenPartDialog->Values.Part];
				LLines->Add(LName.Name);
			}
			LValues.Text = LLines->Text;
			LValues.Count = LLines->Count;
		}
		OrenFreeEditDialog->Values = LValues;
	}
	if(!OrenFreeEditDialog->Execute()) {
		Abort();
	}
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		LCommand->Invoker->Commands->Capacity = Control->SelCount;
		TAutoPointer<TStrings> LLines = new TStringList();
		LLines->Text = OrenFreeEditDialog->Values.Text;
		for(int LI = 0, LRest = Control->SelCount; LI < Control->Count && LRest > 0; ++LI) {
			if(!Control->Items[LI]->Selected) {
				continue;
			}
			--LRest;
			TRenameItem* LItem = static_cast<TRenameItem*>(Control->Items[LI]->Value);
			TRenameName LName = LItem->NewName;
			LName.Parts[OrenPartDialog->Values.Part] = LLines->Strings[LCommand->Invoker->Commands->Count];
			LCommand->Invoker->Commands->Add(new TSetSubCommand(LItem, LName.Name));
		}
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::Enter(int AIndex, const AnsiString& AValue)
{
	TAutoPointer<TNameEditCommand> LCommand = new TNameEditCommand(Control);
	{
		LCommand->Invoker->Commands->Capacity = 1;
		LCommand->Invoker->Commands->Add(new TSetSubCommand(
			static_cast<TRenameItem*>(Control->Items[AIndex]->Value),
			AValue));
	}
	Invoker->Commands->Add(LCommand.Release());
	Invoker->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::LoadSettings(TMemIniFile* AFile, TIniSection* ASection)
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TNameHandler::SaveSettings(TMemIniFile* AFile, TIniSection* ASection)
{
	// Do nothing
}
//---------------------------------------------------------------------------
} // End namespace Oren_namehandler
//---------------------------------------------------------------------------

